    <!-- ***************************************************************************************************************
    *
    * Macro JiraChangeLog
    *
    **************************************************************************************************************** -->
    <xsl:template match="div[@class='nwXsltMacro_ChangeLog']">     
        <xsl:variable name="jiraQueryUrl">
            <xsl:call-template name="get-jira-query-url">
                <xsl:with-param name="groupId" select="p[@class='nwXsltMacro_ChangeLog_groupId']"/>
                <xsl:with-param name="artifactId" select="p[@class='nwXsltMacro_ChangeLog_artifactId']"/>
            </xsl:call-template>
            <xsl:value-of select="'&amp;status=Closed'"/>
            <xsl:value-of select="'&amp;resolution=Fixed'"/>
        </xsl:variable>     

        <xsl:variable name="versions">
            <xsl:for-each select="document($jiraQueryUrl)/rss/channel/item/fixVersion" >
                <xsl:value-of select="."/>
                <xsl:value-of select="' '"/>
            </xsl:for-each>
        </xsl:variable> 

        <xsl:call-template name="emit-change-log">
            <xsl:with-param name="jiraQueryUrl" select="$jiraQueryUrl"/>
            <xsl:with-param name="versions" select="$versions"/>
        </xsl:call-template>
    </xsl:template>

    <!-- ***************************************************************************************************************
    *
    * Macro JiraKnownBugs
    *
    **************************************************************************************************************** -->
    <xsl:template match="div[@class='nwXsltMacro_KnownBugs']">     
        <xsl:variable name="jiraQueryUrl">
            <xsl:call-template name="get-jira-query-url">
                <xsl:with-param name="groupId" select="p[@class='nwXsltMacro_KnownBugs_groupId']"/>
                <xsl:with-param name="artifactId" select="p[@class='nwXsltMacro_KnownBugs_artifactId']"/>
            </xsl:call-template>
            <xsl:value-of select="'&amp;type=1'"/>
            <xsl:value-of select="'&amp;resolution=Unresolved'"/>
        </xsl:variable>     

        <xsl:for-each select="document($jiraQueryUrl)/rss/channel/item">       
            <xsl:call-template name="emit-issue">
                <xsl:with-param name="issue" select="." />
            </xsl:call-template>                            
        </xsl:for-each>
    </xsl:template>

    <!-- ***************************************************************************************************************
    *
    * Macro JiraKnownRFEs
    *
    **************************************************************************************************************** -->
    <xsl:template match="div[@class='nwXsltMacro_KnownRFEs']">     
        <xsl:variable name="jiraQueryUrl">
            <xsl:call-template name="get-jira-query-url">
                <xsl:with-param name="groupId" select="p[@class='nwXsltMacro_KnownRFEs_groupId']"/>
                <xsl:with-param name="artifactId" select="p[@class='nwXsltMacro_KnownRFEs_artifactId']"/>
            </xsl:call-template>
            <xsl:value-of select="'&amp;type=2'"/>
            <xsl:value-of select="'&amp;resolution=Unresolved'"/>
        </xsl:variable>     

        <xsl:for-each select="document($jiraQueryUrl)/rss/channel/item">       
            <xsl:call-template name="emit-issue">
                <xsl:with-param name="issue" select="." />
            </xsl:call-template>                            
        </xsl:for-each>
    </xsl:template>

    <!-- ***************************************************************************************************************
    *
    * 
    *
    **************************************************************************************************************** -->
    <xsl:template name="emit-change-log">
        <xsl:param name="jiraQueryUrl" />
        <xsl:param name="versions" />
        <xsl:param name="done" select="''" />

        <xsl:variable name="newlist" select="concat(normalize-space($versions), ' ')" />
        <xsl:variable name="version" select="substring-before($newlist, ' ')" />
        <xsl:variable name="remaining" select="substring-after($newlist, ' ')" />

        <xsl:if test="not(contains($done, concat($version, ' ')))">
            <xsl:element name="h3">
                <xsl:value-of select="$version"/>
            </xsl:element>
            <xsl:element name="ul">
                <xsl:for-each select="document($jiraQueryUrl)/rss/channel/item">       
                    <xsl:if test="fixVersion = $version">
                        <xsl:call-template name="emit-issue">
                            <xsl:with-param name="issue" select="." />
                        </xsl:call-template>                            
                    </xsl:if>
                </xsl:for-each>
            </xsl:element>        
        </xsl:if>

        <xsl:if test="$remaining">
            <xsl:call-template name="emit-change-log">
                <xsl:with-param name="jiraQueryUrl" select="$jiraQueryUrl" />
                <xsl:with-param name="versions" select="$remaining" />
                <xsl:with-param name="done">
                    <xsl:value-of select="$version" />
                    <xsl:value-of select="' '" />
                    <xsl:value-of select="$done" />
                </xsl:with-param>
            </xsl:call-template>
        </xsl:if>
    </xsl:template>


    <!-- ***************************************************************************************************************
    *
    * Emits the markup for a single issue.
    *
    **************************************************************************************************************** -->
    <xsl:template name="emit-issue">
        <xsl:param name="issue" />
        
        <xsl:variable name="issueType" select="$issue/type/@id" />
        <xsl:variable name="issuePriority" select="$issue/priority/@id" />
        
        <xsl:element name="li">
            <xsl:element name="i">
                <xsl:attribute name="class">
                    <xsl:value-of select="'nw-icon-issue-type-'"/>
                    <xsl:choose>
                        <xsl:when test="$issueType = 1">
                            <xsl:value-of select="'defect'"/>
                        </xsl:when>
                        <xsl:when test="$issueType = 2">
                            <xsl:value-of select="'new-feature'"/>
                        </xsl:when>
                        <xsl:when test="$issueType = 3">
                            <xsl:value-of select="'task'"/>
                        </xsl:when>
                        <xsl:when test="$issueType = 4">
                            <xsl:value-of select="'improvement'"/>
                        </xsl:when>
                    </xsl:choose>
                </xsl:attribute>
            </xsl:element>
            <xsl:element name="i">
                <xsl:attribute name="class">
                    <xsl:value-of select="'nw-icon-issue-priority-'"/>
                    <xsl:choose>
                        <xsl:when test="$issuePriority = 1">
                            <xsl:value-of select="'blocker'"/>
                        </xsl:when>
                        <xsl:when test="$issuePriority = 2">
                            <xsl:value-of select="'critical'"/>
                        </xsl:when>
                        <xsl:when test="$issuePriority = 3">
                            <xsl:value-of select="'major'"/>
                        </xsl:when>
                        <xsl:when test="$issuePriority = 4">
                            <xsl:value-of select="'minor'"/>
                        </xsl:when>
                        <xsl:when test="$issuePriority = 5">
                            <xsl:value-of select="'trivial'"/>
                        </xsl:when>
                    </xsl:choose>
                </xsl:attribute>
            </xsl:element>

            <xsl:element name="a">
                <xsl:attribute name="href">
                    <xsl:value-of select="$issue/link"/>
                </xsl:attribute>
                <xsl:value-of select="'['"/>
                <xsl:value-of select="$issue/key"/>
                <xsl:value-of select="']'"/>
            </xsl:element>
            <xsl:value-of select="' '"/>

            <!-- FIXME: why doesn't Xalan just escape it automatically ? -->
            <xsl:call-template name="replace-substring">
                <xsl:with-param name="original">
                    <xsl:call-template name="replace-substring">
                        <xsl:with-param name="original">
                            <xsl:call-template name="replace-substring">
                                <xsl:with-param name="original" select="$issue/summary"/>
                                <xsl:with-param name="substring" select="'&amp;'"/>
                                <xsl:with-param name="replacement" select="'&amp;amp;'"/>
                            </xsl:call-template>
                        </xsl:with-param>
                        <xsl:with-param name="substring" select="'&lt;'"/>
                        <xsl:with-param name="replacement" select="'&amp;lt;'"/>
                    </xsl:call-template>
                </xsl:with-param>
                <xsl:with-param name="substring" select="'&gt;'"/>
                <xsl:with-param name="replacement" select="'&amp;gt;'"/>
            </xsl:call-template>
        </xsl:element>

        <xsl:text>&#10;</xsl:text>
    </xsl:template>

    <!-- ***************************************************************************************************************
    *
    * Gets the JIRA query URL.
    *
    **************************************************************************************************************** -->
    <xsl:template name="get-jira-query-url">
        <xsl:param name="groupId" />
        <xsl:param name="artifactId" />
        
        <xsl:variable name="jiraUrl">
            <xsl:call-template name="get-issue-management-url">
                <xsl:with-param name="groupId" select="$groupId"/>
                <xsl:with-param name="artifactId" select="$artifactId"/>
            </xsl:call-template>
        </xsl:variable>     
        <xsl:variable name="jiraBaseUrl">
            <xsl:call-template name="substring-before-last">
                <xsl:with-param name="original" select="$jiraUrl"/>
                <xsl:with-param name="substring" select="'/'"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="projectId">
            <xsl:call-template name="substring-after-last">
                <xsl:with-param name="original" select="$jiraUrl"/>
                <xsl:with-param name="substring" select="'/'"/>
            </xsl:call-template>
        </xsl:variable>

        <xsl:value-of select="$jiraBaseUrl"/>
        <xsl:value-of select="'/sr/jira.issueviews:searchrequest-xml/temp/SearchRequest.xml?'"/>
        <xsl:value-of select="'pid='"/>
        <xsl:value-of select="$projectId"/>
   </xsl:template>